var Model3D = function()
{
	this._vertices = [];
	this._normals = [];
	this._materials = [];
	this._material = [];
	this._uvs = [];
	this._vertexPositionBuffer;
	this._vertexNormalBuffer;
	this._vertexColorBuffer;
	this._textureCoordBuffer;
	this._pos;
	this._rotate;
	this._scale;
	this._visible = true;
	this._useTexture = false;
};

Model3D.prototype =
{
	init : function()
	{
		this._pos = new Vector3D(0,0,0);
		this._rotate = new Vector3D(0,0,0);
		this._scale = new Vector3D(1,1,1);
		this._visible = true;
		this._vertices = new Array();
		this._normals = new Array();
		this._materials = new Array();
		this._material = new Array();
		this._uvs = new Array();
	},
	setVertex : function(x,y,z,nx,ny,nz,material)
	{
		this._vertices.push(x,y,z);
		this._normals.push(nx,ny,nz);
		this._material.push(~~(material));
	},
	setUV : function(u,v)
	{
		this._uvs.push(u,v);
	},
	setMaterial : function(r,g,b,a,dif,amb,emi,spc,power,texture)
	{
		this._materials.push(new Material(r,g,b,a,dif,amb,emi,spc,power,texture));
	},
	getColors : function()
	{
		var colors = new Array();
		for ( i = 0; i < this._material.length; i++ )
		{
			c = this._materials[this._material[i]].getColor();
			colors.push(c[0],c[1],c[2],c[3]);
		}
		return colors;
	},
	getVerticesLength : function()
	{
		return ~~(this._vertices.length/3);
	},
	initBuffers : function()
	{
		this._vertexPositionBuffer = _gl.createBuffer();
		_gl.bindBuffer(_gl.ARRAY_BUFFER, this._vertexPositionBuffer);
		_gl.bufferData(_gl.ARRAY_BUFFER, new Float32Array(this._vertices), _gl.STATIC_DRAW);
		this._vertexPositionBuffer.itemSize = 3;
		this._vertexPositionBuffer.numItems = this.getVerticesLength();

		this._vertexNormalBuffer = _gl.createBuffer();
		_gl.bindBuffer(_gl.ARRAY_BUFFER, this._vertexNormalBuffer);
		_gl.bufferData(_gl.ARRAY_BUFFER, new Float32Array(this._normals), _gl.STATIC_DRAW);
		this._vertexNormalBuffer.itemSize = 3;
		this._vertexNormalBuffer.numItems = this.getVerticesLength();

		this._vertexColorBuffer = _gl.createBuffer();
		_gl.bindBuffer(_gl.ARRAY_BUFFER, this._vertexColorBuffer);
		_gl.bufferData(_gl.ARRAY_BUFFER, new Float32Array(this.getColors()), _gl.STATIC_DRAW);
		this._vertexColorBuffer.itemSize = 4;
		this._vertexColorBuffer.numItems = this.getVerticesLength();

		if ( this._uvs.length )
		{
			this._useTexture = true;
			this._textureCoordBuffer = _gl.createBuffer();
			_gl.bindBuffer(_gl.ARRAY_BUFFER, this._textureCoordBuffer);
			_gl.bufferData(_gl.ARRAY_BUFFER, new Float32Array(this._uvs), _gl.STATIC_DRAW);
			this._textureCoordBuffer.itemSize = 2;
			this._textureCoordBuffer.numItems = this.getVerticesLength();
		}
	},
	draw : function()
	{
		if ( !this._visible )
		{
			return;
		}
		mvPushMatrix();

		this.setVertices();

		_gl.uniform1i(_shaderProgram.u_useBone, false);

		_gl.disableVertexAttribArray(_shaderProgram.a_bone);

		_gl.drawArrays(_gl.TRIANGLES, 0, this.getVerticesLength() );

		mvPopMatrix();
	},
	setVertices : function()
	{
		_mvMatrix.translate(this._pos);
		_mvMatrix.rotateX(degreeToRadian(this._rotate.x));
		_mvMatrix.rotateY(degreeToRadian(this._rotate.y));
		_mvMatrix.rotateZ(degreeToRadian(this._rotate.z));
		_mvMatrix.scale(this._scale);

		_gl.enableVertexAttribArray(_shaderProgram.a_vertexNormal);
		_gl.enableVertexAttribArray(_shaderProgram.a_vertexColor);

		_gl.bindBuffer(_gl.ARRAY_BUFFER, this._vertexPositionBuffer);
		_gl.vertexAttribPointer(_shaderProgram.a_vertexPosition, this._vertexPositionBuffer.itemSize, _gl.FLOAT, false, 0, 0);

		_gl.bindBuffer(_gl.ARRAY_BUFFER,this._vertexNormalBuffer);
		_gl.vertexAttribPointer(_shaderProgram.a_vertexNormal, this._vertexNormalBuffer.itemSize, _gl.FLOAT, false, 0, 0);

		_gl.bindBuffer(_gl.ARRAY_BUFFER, this._vertexColorBuffer);
		_gl.vertexAttribPointer(_shaderProgram.a_vertexColor, this._vertexColorBuffer.itemSize, _gl.FLOAT, false, 0, 0);

		_gl.uniform1i(_shaderProgram.u_useLight, true);
		if ( this._useTexture )
		{
			_gl.bindBuffer(_gl.ARRAY_BUFFER, this._textureCoordBuffer);
			_gl.vertexAttribPointer(_shaderProgram.a_textureCoord, this._textureCoordBuffer.itemSize, _gl.FLOAT, false, 0, 0);

			for ( var i = 0; i < this._materials.length; i++ )
			{
				this._materials[i].setTexture(i);
				this._materials[i].setLight();
			}
			_gl.enableVertexAttribArray(_shaderProgram.a_textureCoord);
		}
		else
		{
			_gl.disableVertexAttribArray(_shaderProgram.a_textureCoord);
		}
		_gl.uniform1i(_shaderProgram.u_useTexture, this._useTexture);

		setMatrixUniforms(_pMatrix._e);
	},
	dirRadian : function(dest)
	{
		var radian;
		var dir = new Vector3D(0,0,0);

		dir = dest.subtract(this._pos);

		if (dir.x > 0.0)
		{
			radian = -Math.atan(dir.z / dir.x) + Math.PI / 2;
		}
		else if (dir.x < 0.0)
		{
			radian = -Math.atan(dir.z / dir.x) - Math.PI / 2;
		}
		else
		{
			radian = -Math.atan(dir.z / 0.001) + Math.PI / 2;
		}

		return radian;
	},
	dirDegree : function(dest)
	{
		var degree;

		degree = radianToDegree(this.dirRadian(dest));

		if ( degree < 0 )
		{
			degree += 360;
		}
		if ( degree >= 360 )
		{
			degree -= 360;
		}

		return degree;
	},
	forward : function(delta)
	{
		var pos = new Vector3D(0,0,0);
		var radian = degreeToRadian(this._rotate.y);

		pos.x = this._pos.x + Math.sin(radian) * delta * _elapsed;
		pos.y = this._pos.y;
		pos.z = this._pos.z + Math.cos(radian) * delta * _elapsed;

		return pos;
	},
	forward2 : function(delta)
	{
		var pos = new Vector3D(0,0,0);
		var radian = degreeToRadian(this._rotate.y);

		pos.x = this._pos.x + Math.sin(radian) * delta;
		pos.y = this._pos.y;
		pos.z = this._pos.z + Math.cos(radian) * delta;

		return pos;
	},
	addRotateY : function(delta)
	{
		this._rotate.y += delta * _elapsed;

		if ( this._rotate.y < 0 )
		{
			this._rotate.y += 360;
		}
		if ( this._rotate.y >= 360 )
		{
			this._rotate.y -= 360;
		}
	},
	addRotateY2 : function(delta)
	{
		this._rotate.y += delta;

		if ( this._rotate.y < 0 )
		{
			this._rotate.y += 360;
		}
		if ( this._rotate.y >= 360 )
		{
			this._rotate.y -= 360;
		}
	}
};
